package com.ruoyi.common.utils.file;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.exception.file.FileNameLengthLimitExceededException;
import com.ruoyi.common.exception.file.FileSizeLimitExceededException;
import com.ruoyi.common.exception.file.InvalidExtensionException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.uuid.Seq;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.project.config.AppConfig;
import org.apache.commons.io.FilenameUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Paths;
import java.util.Objects;

/**
 * 文件上传工具类,适用于 Koyo 资料库
 * 
 * @author fangk 
 */
public final class FileUploadKoyoUtils {

    /**
     * 默认大小 50M
     */
    public static final long DEFAULT_MAX_SIZE = 50 * 1024 * 1024;

    /**
     * 默认的文件名最大长度 100
     */
    public static final int DEFAULT_FILE_NAME_LENGTH = 100;

    /**
     * 回收站内的临时文件夹
     */
    public static final String TEMP = "temp";

    /**
     * 根据文件路径上传
     *
     * @param baseDir 相对应用的基目录
     * @param file 上传的文件
     * @return 文件名称
     * @throws IOException
     */
    public static String upload(String baseDir, MultipartFile file, String basePath) throws IOException
    {
        try
        {
            return upload(baseDir, file, MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION, basePath);
        }
        catch (Exception e)
        {
            throw new IOException(e.getMessage(), e);
        }
    }

    /**
     * 文件上传
     *
     * @param baseDir 相对应用的基目录
     * @param file 上传的文件
     * @param allowedExtension 上传文件类型
     * @return 返回上传成功的文件名
     * @throws FileSizeLimitExceededException 如果超出最大大小
     * @throws FileNameLengthLimitExceededException 文件名太长
     * @throws IOException 比如读写文件出错时
     * @throws InvalidExtensionException 文件校验异常
     */
    public static String upload(String baseDir, MultipartFile file, String[] allowedExtension, String basePath)
            throws FileSizeLimitExceededException, IOException, FileNameLengthLimitExceededException,
            InvalidExtensionException
    {
        int fileNameLength = Objects.requireNonNull(file.getOriginalFilename()).length();
        if (fileNameLength > FileUploadKoyoUtils.DEFAULT_FILE_NAME_LENGTH)
        {
            throw new FileNameLengthLimitExceededException(FileUploadKoyoUtils.DEFAULT_FILE_NAME_LENGTH);
        }

        assertAllowed(file, allowedExtension);

        String fileName = extractFilename(file);
        fileName = fileName.replaceAll(" ", "");

        String absPath = getAbsoluteFile(baseDir, fileName).getAbsolutePath();
        file.transferTo(Paths.get(absPath));
        return getPathFileName(baseDir,basePath, fileName);
    }

    /**
     * 编码文件名
     */
    public static String extractFilename(MultipartFile file)
    {
        //return StringUtils.format("{}/{}_{}.{}", DateUtils.datePath(),
        //        FilenameUtils.getBaseName(file.getOriginalFilename()), Seq.getId(Seq.uploadSeqType), getExtension(file));
            return StringUtils.format("{}_{}.{}", FilenameUtils.getBaseName(file.getOriginalFilename()), Seq.getId(Seq.uploadSeqType), getExtension(file));
    }

    public static File getAbsoluteFile(String uploadDir, String fileName) throws IOException
    {
        return FileUploadUtils.getAbsoluteFile(uploadDir, fileName);
    }

    public static String getPathFileName(String uploadDir, String basePath, String fileName) throws IOException
    {
        //int dirLastIndex = basePath.length() + 1;
        //String currentDir = StringUtils.substring(uploadDir, dirLastIndex);
        //return Constants.RESOURCE_PREFIX + "/" + currentDir + "/" + fileName;
        
        String replace = uploadDir.replace(basePath, "");
        return Constants.TRAIN_RESOURCE_PREFIX + "/" + replace + "/" + fileName;
    }

    /**
     * 文件大小校验
     *
     * @param file 上传的文件
     * @return
     * @throws FileSizeLimitExceededException 如果超出最大大小
     * @throws InvalidExtensionException
     */
    public static void assertAllowed(MultipartFile file, String[] allowedExtension)
            throws FileSizeLimitExceededException, InvalidExtensionException
    {
        long size = file.getSize();
        if (size > DEFAULT_MAX_SIZE)
        {
            throw new FileSizeLimitExceededException(DEFAULT_MAX_SIZE / 1024 / 1024);
        }

        String fileName = file.getOriginalFilename();
        String extension = getExtension(file);
        if (allowedExtension != null && !isAllowedExtension(extension, allowedExtension))
        {
            if (allowedExtension == MimeTypeUtils.IMAGE_EXTENSION)
            {
                throw new InvalidExtensionException.InvalidImageExtensionException(allowedExtension, extension,
                        fileName);
            }
            else if (allowedExtension == MimeTypeUtils.FLASH_EXTENSION)
            {
                throw new InvalidExtensionException.InvalidFlashExtensionException(allowedExtension, extension,
                        fileName);
            }
            else if (allowedExtension == MimeTypeUtils.MEDIA_EXTENSION)
            {
                throw new InvalidExtensionException.InvalidMediaExtensionException(allowedExtension, extension,
                        fileName);
            }
            else if (allowedExtension == MimeTypeUtils.VIDEO_EXTENSION)
            {
                throw new InvalidExtensionException.InvalidVideoExtensionException(allowedExtension, extension,
                        fileName);
            }
            else
            {
                throw new InvalidExtensionException(allowedExtension, extension, fileName);
            }
        }
    }

    /**
     * 判断MIME类型是否是允许的MIME类型
     *
     * @param extension allowedExtension
     * @param allowedExtension  allowedExtension
     * @return
     */
    public static boolean isAllowedExtension(String extension, String[] allowedExtension)
    {
        return FileUploadUtils.isAllowedExtension(extension, allowedExtension);
    }

    /**
     * 获取文件名的后缀
     *
     * @param file 表单文件
     * @return 后缀名
     */
    public static String getExtension(MultipartFile file)
    {
        return FileUploadUtils.getExtension(file);
    }

    /**
     * 根据资料库文件(TRAIN_RESOURCE_PREFIX)路径获取本地文件 路径地址
     */
    public static String getLocalPath(String path) {
        return path.replace(Constants.TRAIN_RESOURCE_PREFIX, new File(AppConfig.getDataFolder()).getPath());
    }
    /**
     * 根据资料库文件(TRAIN_RESOURCE_PREFIX)路径获取本地文件 File
     */
    public static File getLocalFile(String path) {
        String localPath = getLocalPath(path);
        return new File(localPath);
    }

    public static String getTempPath(String fileName) {
        if (StringUtils.isEmpty(fileName)) {
            throw new IllegalArgumentException("文件名不能为空");
        }
        
        // 如果fileName有后缀名，去掉后缀名
        if (fileName.contains(".")) {
            fileName = fileName.substring(0, fileName.lastIndexOf("."));
        }
        
        // "file://" + 
        return AppConfig.getRecyclePath() + File.separator + TEMP + File.separator + fileName + ".pdf";
    }

    public static String getInternalPath(String fileName) {
        if (StringUtils.isEmpty(fileName)) {
            throw new IllegalArgumentException("文件名不能为空");
        }
        
        // 如果fileName有后缀名，去掉后缀名
        if (fileName.contains(".")) {
            fileName = fileName.substring(0, fileName.lastIndexOf("."));
        }
        
        return new File(Constants.TRAIN_RESOURCE_PREFIX + File.separator + AppConfig.getRecycleFolder() + File.separator + TEMP + File.separator + fileName + ".pdf").getPath();
    }
}
